// XPathを取得する
function getXPath(node) {
  if (node === document.body) return "/html/body";
  var paths = [];
  // 親要素を遡って#documentになるまで
  while (node && node.nodeName.toLowerCase()!="#document") {
    var index = 0;
    var flag = false;
    // 兄弟要素を遡って検索(xpathでdiv[3]等の場合3を取得する必要があるため、それをindexに格納する)
    for (var sibling = node.previousSibling; sibling; sibling = sibling.previousSibling) {
      if (sibling.nodeName == node.nodeName) {
        index++;
      }
    }
    // 0の場合はその後の兄弟要素に同じnodeがないか確認(あった場合(1)なかったらなにもない)
    if(index===0){
      for (var sibling = node.nextSibling; sibling; sibling = sibling.nextSibling) {
        if (sibling.nodeName == node.nodeName) {
          flag=true;
          break;
        }
      }
    }
    // xpathを結合する
    var tagName = node.nodeName.toLowerCase();
    if(tagName === "#text") tagName = "text()";
    var pathIndex = (index > 0 ? "[" + (index+1) + "]" : index===0 && flag ? "[" + (1) + "]" : "");
    paths.unshift(tagName + pathIndex);
    node = node.parentNode;
  }
  return "/"+paths.join("/");
}


// ロード後に実行する関数
function excute() {
  // すべてのxpathを再帰的に取得する関数
  function getAllXPaths(node) {
    // xpathの取得
    var xpath = getXPath(node);
    // テキストの取得
    var content = getAllTextContent(node);
    
    var xpathObj = {
      "xpath": xpath,
      "content": content
    };
    xpathList.push(xpathObj);
    //var children = node.childNodes; #textまで含めたい場合
    var children = node.children;
    for (var i = 0; i < children.length; i++) {
      var child = children[i];
      getAllXPaths(child);
    }
  }

  getAllXPaths(document.body);
}

// ロードしてから1秒後にxpathを取得する
window.addEventListener('load', function() {
  window.setTimeout(excute, 1000);
});

// テキストを取得する関数
function getAllTextContent(node) {
  var textContent = '';
  // テキストノードの場合は結合
  if (node.nodeType === Node.TEXT_NODE) {
    textContent += node.textContent;
  // 異なる場合は再帰的に子要素を調べてテキストノードを取得する
  } else {
    var childNodes = node.childNodes;
    for (var i = 0; i < childNodes.length; i++) {
      // 再帰的に探索
      textContent += getAllTextContent(childNodes[i]);
    }
  }

  return textContent;
}

// xpathを取得する関数(クリックターゲット用)
function getXpath(element) {
  if(element && element.parentNode) {
    let xpath;
    if(element.tagName === 'font' || element.tagName === 'FONT'){ // fontタグは日本語化で生成されるため元のタグを検索するのに必要ない
      xpath = getXpath(element.parentNode);
    }else{
      xpath = getXpath(element.parentNode) + '/' + element.tagName;

      var s = [];

      for(var i = 0; i < element.parentNode.childNodes.length; i++) {
        var e = element.parentNode.childNodes[i];
        if(e.tagName == element.tagName) {
          s.push(e);
        }
      }

      if(1 < s.length) {
        for(var i = 0; i < s.length; i++) {
          if(s[i] === element) {
            xpath += '[' + (i+1) + ']';
            break;
          }
        }
      }
    }
    return xpath.toLowerCase();
  } else {
    return '';
  }
}


// クリックイベント
document.addEventListener('click', function(e){
  // クリックターゲットのxpath取得
  const targetXpath = getXpath(e.target);
  // ターゲットがあって、xpath配列に格納されているターゲットのxpathが空白でない場合
  if(e.target && xpathList.find(path=>path.xpath===targetXpath).content != ""){
    // ツールチップを表示する
    tooltip.show(Object.assign({},xpathList.find(path=>path.xpath===targetXpath)).content, e.target);
  }
});


// ツールチップクラス
class Tooltip
{
  constructor(){
    // 内容
    this.content = document.createElement( 'code' );
    this.content.className = 'tooltip-content';
    // 影
    this.shadow = document.createElement( 'div' );
    this.shadow.className = 'tooltip-shadow';

    this.shadow.appendChild( this.content );
    if( this.shadow.parentNode != document.body )
    {
      // ドキュメントのbody要素に追加する
      document.body.appendChild( this.shadow );
    }else{

    }
    this.shadow.style.visibility = 'hidden';
  }

  // ツールチップを表示する関数
  show( text, el ){
    // 内容
    while( this.content.hasChildNodes() )
    {
        this.content.removeChild( this.content.lastChild );
    }
    this.content.appendChild( document.createTextNode( text ) );

    // fontタグの場合はfont出ないところまでさかのぼって要素を取得する
    const targetEl = fontParentSearch(el);
    // クリックした要素の座標を取得する
    const targetRect = targetEl.getBoundingClientRect();

    let x = targetRect.left;
    let y = targetRect.top;
    // マウスポインタの位置をドュメント座標に正する
    x += window.pageXOffset || document.documentElement.scrollLeft;
    y += window.pageYOffset || document.documentElement.scrollTop;
    // ツールチップを表示する位置を計算する
    this.shadow.style.left = x + 'px';
    this.shadow.style.top = y - this.shadow.offsetHeight  + 'px';
    this.shadow.style.width = targetRect.width  + 'px';
    this.shadow.style.zIndex = 10000;  
    // 影

    this.shadow.style.visibility = 'visible';



  }

  // ツールチップを隠す関数
  hide(){
    this.shadow.style.visibility = 'hidden';
  }
}

// fontタグを遡って検索する関数
function fontParentSearch(el){
  let targetEl = el;
  if(targetEl.tagName === "font" || targetEl.tagName === "FONT"){
    targetEl = fontParentSearch(el.parentNode);
  }
  return targetEl;
}

var xpathList = []; //xpathを格納しておく配列
let tooltip = new Tooltip(); // tooltipを作成
